__author__    = 'Simone Margaritelli'
__email__     = 'evilsocket@gmail.com'
__copyright__ = 'Copyright 2019, Simone Margaritelli'
__credits__   = ['Nico Chiaraviglio', 'waxwing']
__license__   = 'GPL'
__version__   = '1.6.1'
__status__    = 'Production'

banner = """
                  ,.::XXXXXXX*:.,,,,                                                 
              .:(XXX(`        ` `*XXXX&:.,                                           
         ,.:(XX*'`                    `)*XXX::...,                                   
       *&X* `                              `#XXXXXXX*,.                              
    .X#``     .:&XX*#*:.,                       ``^:*XXXX&:.                         
  .*X`    ,:XXXXXXXXXXXXXX#.                           `^XXXX&,                      
.:X`    .XXXXXXXXXXXXXXXXXX*.                             `^XX&&&.,                  
:`     XXXXXXXXXXXXXXXXXXXXX                   .,            `XXXXX:.                
      XXXXXXXXXXXXXXXXXXXXXXX                .XXXXXX#,           `XXXXX,             
     XXXXXXXXXXXXXXXXXXXXXXXX,               .&XXX, `^X#:.          `^&XX.           
    &XXXXXXXXXXXXXXXXXX&`   `XX,             .:XXXX      `^X*.,        `^XX.         
   #XXXXXXXXXXXXXXXX&`        `XX,            `XXXX,.XXXXXXXXXXX&.,      `XX&        
   XXXXXXXXXXXXXXX(`    .u      `XXX*:.         `*X*.XXXXXXXXXXXXXXX\.    `XX&.      
  ,XXXXXXXXXXXXXX`   ud8888.        `'\*XXX&.      `*XX,XXXXXXXXXXXXXXX&,   `XX&     
  *XXXXXXXXXXXX`   :888'8888.               `XXX.     `*XX&X^`XXXXXXXXXX*"X**XXX&    
  XXXXXXXXXXX`     d888 '88*"   .u    .        `(X#,     `X&,  `XXXXXXXXX  `XXXXX&.  
  XXXXXXXX(`       8888.+"     .d88B :@8c         `XX      `X    XXXXXXXX    `XXXX&. 
 .XXXXXX:`         8888L      ="8888f8888r          `X*,     X   'XXXXXXX,      `"`  
 *XXX:`            '8888c. .+   4888>'88"      uL     `X*   &X   XXXXXXXX&,          
 XX`                "88888*     4888> '    .ue888Nc..    "*X*`   "XXX^XXX&:,         
&X`                   "YP'      4888>     d88E`"888E`       u.      `   `"`          
`                              .d888L .+  888E  888E   ...ue888b                     
                               ^"8888*"   888E  888E   888R Y888r                    
         ergo  %10s         "Y"     888E  888E   888R I888>                    
         keras %10s                 888& .888E   888R I888>                    
         tf    %10s                 *888" 888&   888R I888>                    
                                           `"   "888E  8888cJ8WX                     
                                          .dWi   `88E  "*888*P"                      
                                          4888~  J8*     'Y"                         
                                           ^"===*"`                                  
"""
